@@ -14,94 +14,92 @@ typedef struct semanage_seuser record_t;
 #include "database.h"
 #include "debug.h"
 
-int semanage_seuser_modify_local(
-	semanage_handle_t* handle,
-	const semanage_seuser_key_t* key,
-	const semanage_seuser_t* data) {
+int semanage_seuser_modify_local(semanage_handle_t * handle,
+				 const semanage_seuser_key_t * key,
+				 const semanage_seuser_t * data)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);	
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_modify(handle, dconfig, key, data);
 }
 
-int semanage_seuser_del_local(
-	semanage_handle_t* handle,
-	const semanage_seuser_key_t* key) {
+int semanage_seuser_del_local(semanage_handle_t * handle,
+			      const semanage_seuser_key_t * key)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_del(handle, dconfig, key);
 }
 
-int semanage_seuser_query_local(
-	semanage_handle_t* handle,
-	const semanage_seuser_key_t* key,
-	semanage_seuser_t** response) {
+int semanage_seuser_query_local(semanage_handle_t * handle,
+				const semanage_seuser_key_t * key,
+				semanage_seuser_t ** response)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_query(handle, dconfig, key, response);
 }
 
-int semanage_seuser_exists_local(
-	semanage_handle_t* handle,
-	const semanage_seuser_key_t* key,
-	int* response) {
+int semanage_seuser_exists_local(semanage_handle_t * handle,
+				 const semanage_seuser_key_t * key,
+				 int *response)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_exists(handle, dconfig, key, response);
 }
 
-int semanage_seuser_count_local(
-	semanage_handle_t* handle,
-	unsigned int* response) {
+int semanage_seuser_count_local(semanage_handle_t * handle,
+				unsigned int *response)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_count(handle, dconfig, response);
 }
 
-int semanage_seuser_iterate_local(
-	semanage_handle_t* handle,
-	int (*handler) (
-		const semanage_seuser_t* record,
-		void* varg),
-	void* handler_arg) {
+int semanage_seuser_iterate_local(semanage_handle_t * handle,
+				  int (*handler) (const semanage_seuser_t *
+						  record, void *varg),
+				  void *handler_arg)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_iterate(handle, dconfig, handler, handler_arg);
 }
+
 hidden_def(semanage_seuser_iterate_local)
 
-int semanage_seuser_list_local(
-	semanage_handle_t* handle,
-	semanage_seuser_t*** records,
-	unsigned int* count) {
+int semanage_seuser_list_local(semanage_handle_t * handle,
+			       semanage_seuser_t *** records,
+			       unsigned int *count)
+{
 
-	dbase_config_t* dconfig = semanage_seuser_dbase_local(handle);
+	dbase_config_t *dconfig = semanage_seuser_dbase_local(handle);
 	return dbase_list(handle, dconfig, records, count);
 }
 
 struct validate_handler_arg {
-	semanage_handle_t* handle;
-	const sepol_policydb_t* policydb;
+	semanage_handle_t *handle;
+	const sepol_policydb_t *policydb;
 };
 
-static int validate_handler(
-	const semanage_seuser_t* seuser,
-	void* varg) {
+static int validate_handler(const semanage_seuser_t * seuser, void *varg)
+{
 
-	semanage_user_t* user = NULL;
-	semanage_user_key_t* key = NULL;
+	semanage_user_t *user = NULL;
+	semanage_user_key_t *key = NULL;
 	int exists, mls_ok;
 
 	/* Unpack varg */
-	struct validate_handler_arg* arg = 
-		(struct validate_handler_arg*) varg;
-	semanage_handle_t* handle = arg->handle;
-	const sepol_policydb_t* policydb = arg->policydb;
+	struct validate_handler_arg *arg = (struct validate_handler_arg *)varg;
+	semanage_handle_t *handle = arg->handle;
+	const sepol_policydb_t *policydb = arg->policydb;
 
 	/* Unpack seuser */
-	const char* name = semanage_seuser_get_name(seuser);
-	const char* sename = semanage_seuser_get_sename(seuser);
-	const char* mls_range = semanage_seuser_get_mlsrange(seuser);
-	const char* user_mls_range;
+	const char *name = semanage_seuser_get_name(seuser);
+	const char *sename = semanage_seuser_get_sename(seuser);
+	const char *mls_range = semanage_seuser_get_mlsrange(seuser);
+	const char *user_mls_range;
 
 	/* Make sure the (SElinux) user exists */
 	if (semanage_user_key_create(handle, sename, &key) < 0)
@@ -115,7 +113,7 @@ static int validate_handler(
 
 	/* Verify that the mls range is valid, and that it's contained
 	 * within the (SELinux) user mls range. This range is optional */
-	if (mls_range && sepol_policydb_mls_enabled(policydb)) {		
+	if (mls_range && sepol_policydb_mls_enabled(policydb)) {
 
 		if (semanage_user_query(handle, key, &user) < 0)
 			goto err;
@@ -123,20 +121,20 @@ static int validate_handler(
 
 		if (sepol_mls_check(handle->sepolh, policydb, mls_range) < 0)
 			goto invalid;
-		if (sepol_mls_contains(handle->sepolh, policydb, 
-			user_mls_range, mls_range, &mls_ok) < 0)
+		if (sepol_mls_contains(handle->sepolh, policydb,
+				       user_mls_range, mls_range, &mls_ok) < 0)
 			goto err;
 
 		if (!mls_ok) {
 			ERR(handle, "MLS range %s for Unix user %s "
-			     "exceeds allowed range %s for SELinux user %s",
-			      mls_range, name, user_mls_range, sename);
+			    "exceeds allowed range %s for SELinux user %s",
+			    mls_range, name, user_mls_range, sename);
 			goto invalid;
 		}
 
 	} else if (mls_range) {
 		ERR(handle, "MLS is disabled, but MLS range %s "
-			"was found for Unix user %s", mls_range, name);
+		    "was found for Unix user %s", mls_range, name);
 		goto invalid;
 	}
 
@@ -144,19 +142,19 @@ static int validate_handler(
 	semanage_user_free(user);
 	return 0;
 
-	err:
+      err:
 	ERR(handle, "could not check if seuser mapping for %s is valid", name);
 	semanage_user_key_free(key);
 	semanage_user_free(user);
 	return -1;
 
-	invalid:
+      invalid:
 	if (mls_range)
 		ERR(handle, "seuser mapping [%s -> (%s, %s)] is invalid",
-			name, sename, mls_range);
-	else 
+		    name, sename, mls_range);
+	else
 		ERR(handle, "seuser mapping [%s -> %s] is invalid",
-			name, sename);
+		    name, sename);
 	semanage_user_key_free(key);
 	semanage_user_free(user);
 	return -1;
@@ -166,9 +164,9 @@ static int validate_handler(
  * it will (1) deadlock, because iterate is not reentrant outside
  * a transaction, and (2) be racy, because it makes multiple dbase calls */
 
-int hidden semanage_seuser_validate_local(
-	semanage_handle_t* handle,	
-	const sepol_policydb_t* policydb) {
+int hidden semanage_seuser_validate_local(semanage_handle_t * handle,
+					  const sepol_policydb_t * policydb)
+{
 
 	struct validate_handler_arg arg;
 	arg.handle = handle;
